@@ -2,7 +2,7 @@
 	cuba.h
 		Prototypes for the Cuba library
 		this file is part of Cuba
-		last modified 2 Mar 06 th
+		last modified 12 Feb 10 th
 */
 
 #ifdef __cplusplus
@@ -92,6 +92,10 @@ extern int llvegasnbatch;
 extern int llvegasgridno;
 extern char llvegasstate[128];
 
+extern struct {
+  unsigned int level, seed;
+} cubarandom;
+
 #ifdef __cplusplus
 }
 #endif
